@@ -41,11 +41,18 @@ class _GSettingsSchema:
             for schema in dom.getElementsByTagName("schema"):
                 if schema_name == schema.getAttribute("id"):
                     for key in schema.getElementsByTagName("key"):
+                        #summary is compulsory, description is optional
+                        summary = key.getElementsByTagName("summary")[0].childNodes[0].data
+                        try:
+                            description = key.getElementsByTagName("description")[0].childNodes[0].data
+                        except:
+                            description = ""
                         self._schema[key.getAttribute("name")] = {
-                                "summary"       :   key.getElementsByTagName("summary")[0].childNodes[0].data,
-                                "description"   :   key.getElementsByTagName("description")[0].childNodes[0].data}
+                                "summary"       :   summary,
+                                "description"   :   description
+                        }
         except:
-            logging.critical("Error parsing schema", exc_info=True)
+            logging.critical("Error parsing schema %s (%s)" % (schema_name, schema_path), exc_info=True)
 
     def __repr__(self):
         return "<gtweak.gsettings._GSettingsSchema: %s>" % self._schema_name
@@ -61,6 +68,10 @@ class GSettingsSetting(Gio.Settings):
 
         self._schema = _SCHEMA_CACHE[schema_name]
 
+    def _setting_check_is_list(self, key):
+        variant = Gio.Settings.get_value(self, key)
+        return variant.get_type_string() == "as"
+
     def schema_get_summary(self, key):
         return self._schema._schema[key]["summary"]
         
@@ -70,13 +81,42 @@ class GSettingsSetting(Gio.Settings):
     def schema_get_all(self, key):
         return self._schema._schema[key]
 
-    def get_value(self, key):
-        return Gio.Settings.get_value(self,key).unpack()
+    def setting_add_to_list(self, key, value):
+        """ helper function, ensures value is present in the GSettingsList at key """
+        assert self._setting_check_is_list(key)
 
-    def set_value(self, key, value):
-        Gio.Settings.set_value(self, key, GLib.Variant('s', value))
+        vals = self[key]
+        if value not in vals:
+            vals.append(value)
+            self[key] = vals
+            return True
+
+    def setting_remove_from_list(self, key, value):
+        """ helper function, removes value in the GSettingsList at key (if present)"""
+        assert self._setting_check_is_list(key)
+
+        vals = self[key]
+        try:
+            vals.remove(value)
+            self[key] = vals
+            return True
+        except ValueError:
+            #not present
+            pass
+
+    def setting_is_in_list(self, key, value):
+        assert self._setting_check_is_list(key)
+        return value in self[key]
 
 if __name__ == "__main__":
+    gtweak.GSETTINGS_SCHEMA_DIR = "/usr/share/glib-2.0/schemas/"
+
     key = "draw-background"
     s = GSettingsSetting("org.gnome.desktop.background")
     print s.schema_get_summary(key), s.schema_get_description(key)
+
+    key = "disabled-extensions"
+    s = GSettingsSetting("org.gnome.shell")
+    assert s.setting_add_to_list(key, "foo")
+    assert s.setting_remove_from_list(key, "foo")
+    assert not s.setting_remove_from_list(key, "foo")
